/* GNU Mailutils -- a suite of utilities for electronic mail
   Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif  

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>  
#include <string.h>  
#include <sieve.h>

void
sieve_require (list_t slist)
{
  int status;
  iterator_t itr;
  
  status = iterator_create (&itr, slist);
  if (status)
    {
      sieve_compile_error (sieve_filename, sieve_line_num,
                   _("cannot create iterator: %s"),
		   mu_strerror (status));
      return;
    }

  for (iterator_first (itr); !iterator_is_done (itr); iterator_next (itr))
    {
      char *name;
      int (*reqfn) __PMT ((sieve_machine_t mach, const char *name)) = NULL;
      char *text = NULL;
      
      iterator_current (itr, (void **)&name);

      if (strncmp (name, "comparator-", 11) == 0)
	{
	  name += 11;
	  reqfn = sieve_require_comparator;
	  text = _("required comparator");
	}
      else if (strncmp (name, "test-", 5)  == 0) /* GNU extension */
	{
	  name += 5;
	  reqfn = sieve_require_test;
	  text = _("required test");
	}
      else if (strcmp (name, "relational") == 0) /* RFC 3431 */
	{
	  reqfn = sieve_require_relational;
	  text = "";
	}
      else
	{
	  reqfn = sieve_require_action;
	  text = _("required action");
	}

      if (reqfn (sieve_machine, name))
	{
	  sieve_compile_error (sieve_filename, sieve_line_num,
			       _("source for the %s %s is not available"),
			       text,
			       name);
	}
    }
  iterator_destroy (&itr);
}
     
